#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

double sq(double x)
  {
  return(x*x);
  }

int GaussJordan(double*A,double*B,int n)
  {
  int i,j,k,ip,jp,*pivot;
  double a,*C,p,b;
  if(n<1)
    return(__LINE__);
  if(n==1)
    {
    a=A[0];
    if(fabs(a)<FLT_MIN)
      return(__LINE__);
    B[0]/=a;
    return(0);
    }
  if((pivot=calloc(n,sizeof(int)))==NULL)
    return(__LINE__);
  for(i=0;i<n;i++)
    pivot[i]=i;
  for(k=0;k<n-1;k++)
    {
    ip=k;
    jp=k;
    p=fabs(A[k*(n+1)]);
    for(i=k;i<n;i++)
      {
      for(j=k;j<n;j++)
        {
        a=fabs(A[n*i+j]);
        if(a>p)
          {
          ip=i;
          jp=j;
          p=a;
          }
        }
      }
    if(p<FLT_MIN)
      {
      free(pivot);
      return(__LINE__);
      }
    if(ip!=k)
      {
      b=B[k];
      B[k]=B[ip];
      B[ip]=b;
      for(j=k;j<n;j++)
        {
        a=A[n*k+j];
        A[n*k+j]=A[n*ip+j];
        A[n*ip+j]=a;
        }
      }
    if(jp!=k)
      {
      j=pivot[jp];
      pivot[jp]=pivot[k];
      pivot[k]=j;
      for(i=0;i<n;i++)
        {
        a=A[n*i+k];
        A[n*i+k]=A[n*i+jp];
        A[n*i+jp]=a;
        }
      }
    for(i=k+1;i<n;i++)
      {
      a=A[n*i+k]/A[n*k+k];
      B[i]-=a*B[k];
      for(j=k+1;j<n;j++)
        A[n*i+j]-=a*A[n*k+j];
      }
    }
  a=A[n*n-1];
  p=fabs(a);
  if(p<FLT_MIN)
    {
    free(pivot);
    return(__LINE__);
    }
  C=A+n*(n-1);
  C[n-1]=B[n-1]/a;
  for(k=1;k<n;k++)
    {
    i=n-1-k;
    b=0;
    for(j=i+1;j<n;j++)
      b+=A[n*i+j]*C[j];
    C[i]=(B[i]-b)/A[n*i+i];
    }
  for(i=0;i<n;i++)
    B[pivot[i]]=C[i];
  free(pivot);
  return(0);
  }

void mult(double*u,double*v,double*w,int l,int m,int n)
  {
  int i,j,k;
  for(i=0;i<l;i++)
    for(j=0;j<n;j++)
      for(w[n*i+j]=k=0;k<m;k++)
        w[n*i+j]+=u[m*i+k]*v[n*k+j];
  }

FILE*fp;

int Broyden( /* Broyden's method for nonlinear least-squares */
  void func(double*,double*),
  double*Xmin,
  double*Xmax,
  double*X,
  int n,
  int m,
  int maxcalls)
  {
  int better,calls,error,i,j,k,step,try;
  double damp=2,fTf,fTf0,fTf1,fTf2,gTg,sTs,t,Xo,Xa,Xb,Xmn,Xmx,xTx0;
  double*B,*BtB,*F,*F1,*F2,*G,*S,*X1,*X2,*Y;
  printf("Broyden's Method\n");
  if(n<1)
    return(__LINE__);
  if(m<n)
    return(__LINE__);
  for(i=0;i<n;i++)
    {
    if(Xmin[i]>=Xmax[i])
      return(__LINE__);
    X[i]=(Xmin[i]+Xmax[i])/2.;
    }
  k=8*n+8*m+n*n+m*n;
  if((X1=calloc(k,sizeof(double)))==NULL)
    return(__LINE__);
  X2=X1+n;
  G =X2+n;
  F =G +n;
  F1=F +m;
  F2=F1+m;
  Y =F2+m;
  B =Y +m;
  BtB=B+m*n;
  S=BtB+n*n;
  calls=0;
  fTf0=1;
  fTf=fTf0;
  xTx0=0.;
  for(i=0;i<n;i++)
    {
    xTx0=xTx0+sq(Xmax[i]-Xmin[i]);
    G[i]=0.;
    fprintf(fp,"%lG ",X[i]);
    }
  func(X,F);
  mult(F,F,&fTf,1,m,1);
  fprintf(fp,"%lG\n",fTf);
  fTf0=fTf;
  try=0;
  t=1.;
  while(1)
    {
    try++;
    t*=2.;
    if(try>16)
      {
      free(X1);
      return(__LINE__);
      }
    step=0;
    memcpy(X1,X,n*sizeof(double));
    memcpy(X2,X,n*sizeof(double));
    memset(B,0,n*m*sizeof(double));
    for(k=0;k<n;k++)
      {
      Xo=X[k];
      calls++;
      if(maxcalls>0&&calls>maxcalls)
        {
        free(X1);
        return(__LINE__);
        }
      Xa=Xo-(Xo-Xmin[k])/t;
      X1[k]=Xa;
      func(X1,F1);
      mult(F1,F1,&fTf1,1,m,1);
      if(fTf1<fTf)
        {
        memcpy(X,X1,n*sizeof(double));
        memcpy(F,F1,m*sizeof(double));
        fTf=fTf1;
        }
      calls++;
      if(maxcalls>0&&calls>maxcalls)
        {
        free(X1);
        return(__LINE__);
        }
      Xb=Xo+(Xmax[k]-Xo)/t;
      X2[k]=Xb;
      func(X2,F2);
      mult(F2,F2,&fTf2,1,m,1);
      if(fTf1<fTf)
        {
        memcpy(X,X1,n*sizeof(double));
        memcpy(F,F1,m*sizeof(double));
        fTf=fTf1;
        }
      for(j=0;j<m;j++)
        Y[j]=F2[j]-F1[j];
      if(fabs(Xb-Xa)<FLT_EPSILON)
        {
        free(X1);
        return(__LINE__);
        }
      for(j=0;j<m;j++)
        B[m*k+j]=Y[j]/(Xb-Xa);
      X1[k]=X2[k]=Xo;
      }
    memcpy(X2,X,n*sizeof(double));
    memcpy(F2,F,m*sizeof(double));
    fTf2=fTf;
    better=0;
    while(1)
      {
      printf("calls=%i, X=%lG",calls,X2[0]);
      fprintf(fp,"%lG",X2[0]);
      for(i=1;i<n;i++)
        {
        printf(",%lG",X2[i]);
        fprintf(fp," %lG",X2[i]);
        }
      printf(", fTf=%lG\n",fTf2);
      fprintf(fp," %lG\n",fTf2);
      memcpy(X1,X2,n*sizeof(double));
      memcpy(F1,F2,m*sizeof(double));
      fTf1=fTf2;
      for(i=0;i<n;i++)
        {
        mult(B+m*i,F2,G+i,1,m,1);
        for(k=0;k<n;k++)
          mult(B+m*i,B+m*k,BtB+n*i+k,1,m,1);
        }
      error=GaussJordan(BtB,G,n);
      if(error)
        {
        free(X1);
        return(error);
        }
      memcpy(S,G,n*sizeof(double));
      for(i=0;i<n;i++)
        {
        Xmn=(X1[i]+Xmin[i])/2.;
        Xmx=(X1[i]+Xmax[i])/2.;
        X2[i]=max(Xmn,min(Xmx,X1[i]-S[i]/damp));
        S[i]=X2[i]-X1[i];
        }
      mult(S,S,&sTs,1,n,1);
      calls++;
      if(maxcalls>0&&calls>maxcalls)
        {
        free(X1);
        return(__LINE__);
        }
      func(X2,F2);
      mult(F2,F2,&fTf2,1,m,1);
      if(fTf2<fTf)
        {
        memcpy(X,X2,n*sizeof(double));
        memcpy(F,F2,m*sizeof(double));
        fTf=fTf2;
        better=1;
        }
      else
        {
        step++;
        if(step>16)
          break;
        }
      for(i=0;i<n;i++)
        mult(B+m*i,F2,G+i,1,m,1);
      mult(G,G,&gTg,1,n,1);
      better=0;
      if(sqrt(fTf)<FLT_MIN*sqrt(fTf0))
        {
        free(X1);
        return(0);
        }
      if(sqrt(sTs)<FLT_MIN*sqrt(xTx0))
        {
        free(X1);
        return(0);
        }
      if(sqrt(gTg)<FLT_MIN*sqrt(fTf0/xTx0))
        {
        free(X1);
        return(0);
        }
      for(j=0;j<m;j++)
        {
        Y[j]=F2[j]-F1[j];
        for(i=0;i<n;i++)
          Y[j]=Y[j]-B[m*i+j]*S[i];
        }
      for(j=0;j<m;j++)
        for(i=0;i<n;i++)
          B[m*i+j]=B[m*i+j]+S[i]*Y[j]/sTs;
      }
    }
  }

typedef struct{double x,y;}DATA;

DATA data1[]={{  0,  0},{  2,106},{  4,178},{  6,294},{  8,318},{ 10,370},
    { 12,442},{ 14,532},{ 16,578},{ 18,622},{ 20,674},{ 22,658},{ 24,708},
    { 26,702},{ 28,724},{ 30,716},{ 32,780},{ 34,782},{ 36,766},{ 38,758},
    { 40,806},{ 42,828},{ 44,814},{ 46,790},{ 48,790},{ 50,762},{ 52,780},
    { 54,748},{ 56,798},{ 58,730},{ 60,700},{ 62,772},{ 64,732},{ 66,704},
    { 68,668},{ 70,650},{ 72,706},{ 74,654},{ 76,634},{ 78,688},{ 80,636},
    { 82,626},{ 84,652},{ 86,562},{ 88,614},{ 90,582},{ 92,576},{ 94,556},
    { 96,514},{ 98,544},{100,466},{102,506},{104,456},{106,430},{108,502},
    {110,502},{112,432},{114,438},{116,468},{118,444},{120,404},{122,438},
    {124,414},{126,366},{128,386},{130,314},{132,392},{134,324},{136,310},
    {138,312},{140,312},{142,298},{144,290},{146,290},{148,308},{150,244},
    {152,216},{154,274},{156,280},{158,196},{160,258},{162,182},{164,178},
    {166,190},{168,188},{170,208},{172,222},{174,200},{176,194},{178,154},
    {180,200},{182,146},{184,158},{186,160},{188,152},{190,164},{192,178},
    {194,150},{196,138},{198,162}};

void var1(double*x,double*a,double*b)
  {
  *a=25.+50.*(1.+tanh(x[0]))/2.;
  *b=25.+50.*(1.+tanh(x[1]))/2.;
  }

void resid1(double*x,double*f)
  {
  int i,n;
  double a,b;
  var1(x,&a,&b);
  n=sizeof(data1)/sizeof(data1[0]);
  for(i=0;i<n;i++)
    f[i]=a*data1[i].x/exp(data1[i].x/b)-data1[i].y;
  }

DATA data2[]={{  0,  0},{  1,  2},{  2, 11},{  3, 15},{  4, 10},{  5,  5},
    {  6, 14},{  7, 32},{  8, 23},{  9, 25},{ 10, 34},{ 11, 46},{ 12, 54},
    { 13, 46},{ 14, 76},{ 15, 67},{ 16, 68},{ 17, 91},{ 18, 89},{ 19,101},
    { 20, 98},{ 21,116},{ 22,118},{ 23,129},{ 24,146},{ 25,138},{ 26,154},
    { 27,162},{ 28,156},{ 29,176},{ 30,179},{ 31,182},{ 32,179},{ 33,186},
    { 34,197},{ 35,191},{ 36,200},{ 37,203},{ 38,208},{ 39,205},{ 40,214},
    { 41,207},{ 42,215},{ 43,209},{ 44,223},{ 45,217},{ 46,223},{ 47,226},
    { 48,222},{ 49,230},{ 50,217},{ 51,220},{ 52,229},{ 53,220},{ 54,220},
    { 55,233},{ 56,221},{ 57,226},{ 58,233},{ 59,219},{ 60,232},{ 61,233},
    { 62,232},{ 63,233},{ 64,225},{ 65,219},{ 66,225},{ 67,221},{ 68,224},
    { 69,222},{ 70,228},{ 71,223},{ 72,229},{ 73,227},{ 74,234},{ 75,226},
    { 76,234},{ 77,224},{ 78,227},{ 79,234},{ 80,225},{ 81,224},{ 82,224},
    { 83,224},{ 84,235},{ 85,224},{ 86,234},{ 87,218},{ 88,223},{ 89,225},
    { 90,229},{ 91,229},{ 92,221},{ 93,225},{ 94,223},{ 95,232},{ 96,224},
    { 97,227},{ 98,237},{ 99,226},{100,224},{101,230},{102,233},{103,230},
    {104,225},{105,226},{106,222},{107,227},{108,226},{109,230},{110,226},
    {111,223},{112,226},{113,224},{114,223},{115,226},{116,223},{117,231},
    {118,231},{119,232},{120,232}};

void var2(double*x,double*a,double*b)
  {
  *a=4.+50.*(1.+tanh(x[0]))/2.;
  *b=0.5+2.5*(1.+tanh(x[1]))/2.;
  }

void resid2(double*x,double*f)
  {
  int i,m,n;
  double a,b,c;
  var2(x,&a,&b);
  n=sizeof(data2)/sizeof(data2[0]);
  for(c=m=0,i=3*n/4;i<n;i++,m++)
    c+=data2[i].y;
  c/=m;
  for(i=0;i<n;i++)
    f[i]=c*tanh(pow(data2[i].x/a,b))-data2[i].y;
  }

void resid3(double*X,double*F)
  {
  F[0]=75.*X[0]*X[0]-169.*X[1]*X[1]/9.;
  F[1]=845.*X[0]*X[1]*X[1]/3.-125.*X[0]*X[0]*X[0]-1.;
  }

void resid4(double*X,double*F)
  {
  double X1,X2,X3;
  X1=X[0]*5.;
  X2=X[1]-0.2;
  X3=-M_PI*X[2]/1.8;
  F[0]=3.*X1-cos(X2*X3)-0.5;
  F[1]=X1*X1-81.*sq(X2+0.1)+sin(X3)+1.;
  F[2]=exp(-X1*X2)+20.*X3+(10.*M_PI-3.)/3.;
  }

void resid5(double*X,double*F)
  {
  double X1,X2,X3;
  X1=X[0]-0.1;
  X2=X[1]/2.;
  X3=X[2]/0.3;
  F[0]=X1+cos(X1*X2*X3)-1.;
  F[1]=pow(fabs(1.-X1),0.25)+X2+0.05*X3*X3-0.15*X3-1.;
  F[2]=-X1*X1-0.1*X2*X2+0.01*X2+X3-1.;
  }

void resid6(double*X,double*F)
  {
  double X1,X2,X3;
  X1=X[0]*8.77129/0.1;
  X2=X[1]*0.259695/0.2;
  X3=X[2]*(-1.37228)/0.3;
  F[0]=X1*exp(X2*1)+X3*1-10.;
  F[1]=X1*exp(X2*2)+X3*2-12.;
  F[2]=X1*exp(X2*3)+X3*3-15.;
  }

void resid7(double*X,double*F)
  {
  double X1,X2,X3,X4;
  X1=X[0]*1.2/0.1;
  X2=X[1]*5.6/0.2;
  X3=X[2]*4.3/0.3;
  X4=X[3]*1.0/0.4;
  F[0]=X1+2.*X2+X3+4.*X4-20.7;
  F[1]=X1*X1+2*X1*X2+X4*X4*X4-15.88;
  F[2]=X1*X1*X1+X3*X3+X4-21.218;
  F[3]=3.*X2+X3*X4-21.1;
  }

void resid8(double*X,double*F)
  {
  double CA,T1,T0,T2;
  double T[]={1,2,3,4,5,6,7,8,9};
  double A[]={2.14737,1.69412,1.2,0.64615,0,-0.8,-1.88571,-3.6,-7.2};
  int i;
  CA= 3.*X[0];
  T1=25.*X[1];
  T0=35.*X[2];
  T2=45.*X[3];
  for(i=0;i<9;i++)
    F[i]=CA*(T[i]-T1)*(T[i]-T2)/(T0-T[i])-A[i];
  }

void ResidualSurface(char*fname,void func(double*,double*),double Xm,double Xx,double Ym,double Yx,int nx,int ny)
  {
  int x,y;
  double p[2],q[2],Z,Zm,Zx;
  FILE*fp;
  printf("creating surface: %s\n",fname);
  if((fp=fopen(fname,"wt"))==NULL)
    {
    printf("can't create output file\n");
    exit(1);
    }
  fprintf(fp,"%i\n",nx);
  for(x=0;x<nx;x++)
    {
    p[0]=Xm+x*(Xx-Xm)/(nx-1);
    fprintf(fp,"%lG\n",p[0]);
    }
  fprintf(fp,"%i\n",ny);
  for(y=0;y<ny;y++)
    {
    p[1]=Ym+y*(Yx-Ym)/(ny-1);
    fprintf(fp,"%lG\n",p[1]);
    }
  fprintf(fp,"%i\n",nx*ny);
  Zm=DBL_MAX;
  Zx=-DBL_MAX;
  for(y=0;y<ny;y++)
    {
    p[1]=Ym+y*(Yx-Ym)/(ny-1);
    for(x=0;x<nx;x++)
      {
      p[0]=Xm+x*(Xx-Xm)/(nx-1);
      func(p,q);
      Z=log(hypot(q[0],q[1]));
      Zm=min(Zm,Z);
      Zx=max(Zx,Z);
      fprintf(fp,"%lG\n",Z);
      }
    printf("\r%.0lf%%",(y+1)*100./ny);
    }
  fclose(fp);
  printf("\r%lG<Z<%lG\n",Zm,Zx);
  }

void ResidualVolume(char*fname,void func(double*,double*),double Xm,double Xx,double Ym,double Yx,double Zm,double Zx,int nx,int ny,int nz)
  {
  int x,y,z;
  double p[3],q[3],W,Wm,Wx;
  FILE*fp;
  printf("creating volume: %s\n",fname);
  if((fp=fopen(fname,"wt"))==NULL)
    {
    printf("can't create output file\n");
    exit(1);
    }
  fprintf(fp,"%i\n",nx);
  for(x=0;x<nx;x++)
    {
    p[0]=Xm+x*(Xx-Xm)/(nx-1);
    fprintf(fp,"%lG\n",p[0]);
    }
  fprintf(fp,"%i\n",ny);
  for(y=0;y<ny;y++)
    {
    p[1]=Ym+y*(Yx-Ym)/(ny-1);
    fprintf(fp,"%lG\n",p[1]);
    }
  fprintf(fp,"%i\n",nz);
  for(z=0;z<nz;z++)
    {
    p[2]=Zm+z*(Zx-Zm)/(nz-1);
    fprintf(fp,"%lG\n",p[2]);
    }
  fprintf(fp,"%i\n",nx*ny*nz);
  Wm=DBL_MAX;
  Wx=-DBL_MAX;
  for(z=0;z<nz;z++)
    {
    p[2]=Zm+z*(Zx-Zm)/(nz-1);
    for(y=0;y<ny;y++)
      {
      p[1]=Ym+y*(Yx-Ym)/(ny-1);
      for(x=0;x<nx;x++)
        {
        p[0]=Xm+x*(Xx-Xm)/(nx-1);
        func(p,q);
        W=sqrt(q[0]*q[0]+q[1]*q[1]+q[2]*q[2]);
        Wm=min(Wm,W);
        Wx=max(Wx,W);
        fprintf(fp,"%lG\n",W);
        }
      }
    printf("\r%.0lf%%",(z+1)*100./nz);
    }
  fclose(fp);
  printf("\r%lG<W<%lG\n",Wm,Wx);
  }

void test12()
  {
  double a,b,X[2];
  double Xmin[2]={-2.,-2.};
  double Xmax[2]={ 2., 2.};
  fp=fopen("resid1.stp","wt");
  Broyden(resid1,Xmin,Xmax,X,2,sizeof(data1)/sizeof(data1[0]),99);
  fclose(fp);
  var1(X,&a,&b);
  printf("a=%lG, b=%lG\n",a,b);
  fp=fopen("resid2.stp","wt");
  Broyden(resid2,Xmin,Xmax,X,2,sizeof(data2)/sizeof(data2[0]),99);
  fclose(fp);
  var2(X,&a,&b);
  printf("a=%lG, b=%lG\n",a,b);
  }

void test345678()
  {
  double X[4];
  double Xmin[4]={0.,0.,0.,0.};
  double Xmax[4]={1.,1.,1.,1.};

  fp=fopen("resid3.stp","wt");
  Broyden(resid3,Xmin,Xmax,X,2,2,99);
  printf("X=%lG,%lG\n",X[0],X[1]);
  ResidualSurface("resid3.tb2",resid3,0.,1.,0.,1.,96,96);
  fclose(fp);

  fp=fopen("resid4.stp","wt");
  ResidualVolume("resid4.tb3",resid4,0.,1.,0.,1.,0.,1.,48,48,48);
  Broyden(resid4,Xmin,Xmax,X,3,3,99);
  printf("X=%lG,%lG,%lG\n",X[0],X[1],X[2]);
  fclose(fp);

  fp=fopen("resid5.stp","wt");
  ResidualVolume("resid5.tb3",resid5,0.,1.,0.,1.,0.,1.,48,48,48);
  Broyden(resid5,Xmin,Xmax,X,3,3,99);
  printf("X=%lG,%lG,%lG\n",X[0],X[1],X[2]);
  fclose(fp);

  fp=fopen("resid6.stp","wt");
  ResidualVolume("resid6.tb3",resid6,0.,1.,0.,1.,0.,1.,48,48,48);
  Broyden(resid6,Xmin,Xmax,X,3,3,99);
  printf("X=%lG,%lG,%lG\n",X[0],X[1],X[2]);
  fclose(fp);

  fp=fopen("resid7.stp","wt");
  Broyden(resid7,Xmin,Xmax,X,4,4,99);
  printf("X=%lG,%lG,%lG,%lG\n",X[0],X[1],X[2],X[3]);
  fclose(fp);

  fp=fopen("resid8.stp","wt");
  Broyden(resid8,Xmin,Xmax,X,4,9,99);
  printf("X=%lG,%lG,%lG,%lG\n",X[0],X[1],X[2],X[3]);
  fclose(fp);
  }


int main(int argc,char**argv,char**envp)
  {
  test12();
  test345678();
  return(0);
  }
