char*gModuleName="\026Mathematical Functions";
/*******************************************************************************

  This Excel Add-In was created by Dudley J. Benton

  Follow these two steps to compile and link the Add-In using Visual Studio:

    cl /O2 /Ot /D "_WINDOWS" /D "_WINDLL" /W3 /WX /c addin.c
    link /DLL addin.obj xlcall32.lib /OUT:"addin32.xll"

*******************************************************************************/

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#define _USE_MATH_DEFINES
#include <float.h>
#include <math.h>
#include <windows.h>
#include <xlcall.h>

/*******************************************************************************
  FUNCTION DEFINITIONS AND ARGUMENTS - ALL FUNCTIONS MUST BE DEFINED HERE

  Each function is defined by a group of entries ending in a NULL strings.
  Each entry is a character, the variable name, and description. The
  character tells what type of variable it is according to this list:

  A boolean (short int 0 or 1) passed by value
  B double passed by value
  C null-terminaed string passed by pointer
  D Pascal (length predecated) string passed by pointer
  E double passed by pointer
  F null-terminaed string passed by pointer
  G Pascal (length predecated) string passed by pointer
  H unsigned short (16-bit) int (i.e., WORD) passed by value
  I signed short (16-bit) int passed by value
  J signed long (32-bit) int passed by value
  K floating point array structure?
  L boolean (short int 0 or 1) passed by pointer
  M signed short (16-bit) int passed by pointer
  N signed long (32-bit) int passed by pointer
  O array structure {WORD,WORD,double*} passed as pointer
  P XLOPER passed as pointer used for variable type worksheet values and arrays
  R XLOPER passed as pointer used for values, arrays, and range references

  ALL FUNCTIONS MUST BE DEFINED HERE
*******************************************************************************/

struct{char type;char*name,*desc;}Funs[]={
  {'R',"Lagrange"    ,"performs Lagrange interpolation"},
  {'R',"xValues"     ,"a range of X values (any order)"},
  {'R',"yValues"     ,"a range of Y values (one for each X)"},
  {'B',"x"           ,"the value of X to be interpolated"},
  { 0 ,NULL          ,NULL},

  {'R',"Spline"      ,"performs cubic spline interpolation"},
  {'R',"xValues"     ,"a range of X values (any order)"},
  {'R',"yValues"     ,"a range of Y values (one for each X)"},
  {'B',"x"           ,"the value of X to be interpolated"},
  { 0 ,NULL          ,NULL},

  {'R',"LLSQ"        ,"calculates coefficients for linear least-squares curve fit without constraints"},
  {'R',"xValues"     ,"a range of X values(in any order)"},
  {'R',"yValues"     ,"a range of Y values(one for each X)"},
  {'J',"terms"       ,"the number of terms (1=constant,2=linear,3=quadratic,4=cubic,...)"},
  { 0 ,NULL          ,NULL},

  {'R',"LLSQLC"      ,"calculates coefficients for linear least-squares curve fit with one or more constraints"},
  {'R',"xValues"     ,"a range of X values (in any order)"},
  {'R',"yValues"     ,"a range of Y values (one for each X value)"},
  {'J',"terms"       ,"the number of terms (1=constant,2=linear,3=quadratic,4=cubic,...)"},
  {'R',"xConstraints","a range of X constraints (one or more,in any order)"},
  {'R',"yConstraints","a range of Y constraints (one for each X constraint)"},
  { 0 ,NULL          ,NULL},

  { 0 ,NULL          ,NULL}};

/*******************************************************************************
  THIS SECTION IS ALWAYS THE SAME
*******************************************************************************/

int APIENTRY DllMain(HANDLE hModule,DWORD fdwReason,LPVOID lpReserved)
  {
  return(1);
  }

char**BuildFunctionParms()
  {
  char args[256],**funs,type[31];
  int i,j,m,n;
  for(m=j=i=0;Funs[i].type;i++,j++)
    {
    for(n=0;Funs[i+1+n].type;n++)
      ;
    i+=n+1;
    if(sizeof(void*)>4)
      m+=n+5;
    else
      m+=min(20,n)+5;
    }
  if((funs=calloc(m+1,sizeof(char*)))==NULL)
    return(NULL);
  for(m=i=0;Funs[i].type;i++)
    {
    funs[m++]=_strdup(Funs[i].name);
    memset(type,0,sizeof(type));
    type[0]=Funs[i].type;
    args[0]=0;
    for(n=0;Funs[i+1+n].type;n++)
      {
      type[n+1]=Funs[i+1+n].type;
      if(n)
        strcat(args,",");
      strcat(args,Funs[i+1+n].name);
      }
    if(n>30)
      return(NULL);
    funs[m++]=_strdup(type);
    funs[m++]=_strdup(args);
    funs[m++]=_strdup(Funs[i].desc);
    for(j=0;j<n;j++)
      {
      if(j==19&&n>20&&sizeof(void*)==4)
        {
        strcpy(args,Funs[i+1+j].desc);
        strcat(args,", etc.");
        funs[m++]=_strdup(args);
        }
      else if(j<20||sizeof(void*)>4)
        funs[m++]=_strdup(Funs[i+1+j].desc);
      }
    m++;
    i+=n+1;
    }
  return(funs);
  }

char**functionParms;

__declspec(dllexport) int xlAutoOpen(void)
  {
  int i,j,k,l,rv;
  XLOPER xConfirm,xDLL,*xStr,**xPtr,xStrT,xIntT;
  if((functionParms=BuildFunctionParms())==NULL)
    {
    xStrT.val.str=" Error: can't build function params!";
    xStrT.val.str[0]=(char)(strlen(xStrT.val.str)-1);
    Excel4(xlcAlert,0,2,&xStrT,&xIntT);
    return(0);
    }
  for(k=i=0;functionParms[i];i++)
    {
    for(j=0;functionParms[i];i++)
      j++;
    k=max(k,j);
    }
  k+=5;
  xStr=calloc(k,sizeof(XLOPER));
  xPtr=calloc(k+1,sizeof(XLOPER*));
  xIntT.xltype=xltypeInt;
  xIntT.val.w=2;
  xStrT.xltype=xltypeStr;
  Excel4(xlGetName,&xDLL,0);
  xPtr[0]=&xDLL;
  for(i=0;functionParms[i];i++)
    {
    for(j=0;functionParms[i];i++)
      {
      l=(int)strlen(functionParms[i]);
      xStr[j].xltype=xltypeStr;
      xStr[j].val.str=malloc(l+1);
      memcpy(xStr[j].val.str+1,functionParms[i],l);
      xStr[j].val.str[0]=l;
      xPtr[j+1]=xStr+j;
      j++;
      if(j==2)
        {
        l=(int)strlen(functionParms[i-1]);
        xStr[j].xltype=xltypeStr;
        xStr[j].val.str=malloc(l+1);
        memcpy(xStr[j].val.str+1,functionParms[i-1],l);
        xStr[j].val.str[0]=l;
        xPtr[j+1]=xStr+j;
        j++;
        }
      if(j==4)
        {
        l=1;
        xStr[j].xltype=xltypeStr;
        xStr[j].val.str=malloc(l+1);
        memcpy(xStr[j].val.str+1,"1",l);
        xStr[j].val.str[0]=l;
        xPtr[j+1]=xStr+j;
        j++;
        }
      if(j==5)
        {
        l=(int)strlen(gModuleName+1);
        xStr[j].xltype=xltypeStr;
        xStr[j].val.str=malloc(l+1);
        memcpy(xStr[j].val.str+1,gModuleName+1,l);
        xStr[j].val.str[0]=l;
        xPtr[j+1]=xStr+j;
        j++;
        }
      if(j==6)
        {
        l=0;
        xStr[j].xltype=xltypeStr;
        xStr[j].val.str=malloc(1);
        xStr[j].val.str[0]=0;
        xPtr[j+1]=xStr+j;
        j++;
        }
      if(j==7)
        {
        l=0;
        xStr[j].xltype=xltypeStr;
        xStr[j].val.str=malloc(1);
        xStr[j].val.str[0]=0;
        xPtr[j+1]=xStr+j;
        j++;
        }
      }
    rv=Excel4v(xlfRegister,&xConfirm,j+1,&xPtr[0]);
    for(k=0;k<j;k++)
      free(xStr[k].val.str);
    if(rv!=xlretSuccess)
      {
      xStrT.val.str=" Error: Failed to register the XLL!";
      xStrT.val.str[0]=(char)(strlen(xStrT.val.str)-1);
      Excel4(xlcAlert,0,2,&xStrT,&xIntT);
      return(0);
      }
    }
  Excel4(xlFree,0,1,&xDLL);
  return(1);
  }

__declspec(dllexport) int xlAutoClose(void)
  {
  return(1);
  }

__declspec(dllexport) LPXLOPER xlAutoRegister(LPXLOPER pxName)
  {
  static XLOPER xRegId;
  xRegId.xltype=xltypeErr;
  xRegId.val.err=xlerrValue;
  return(&xRegId);
  }

__declspec(dllexport) int xlAutoAdd(void)
  {
  static XLOPER xStrT,xIntT;
  char szBuf[255];
  sprintf(szBuf," %s Loaded!\nbuilt on %s at %s",gModuleName+1,__DATE__,__TIME__);
  szBuf[0]=(char)(strlen(szBuf)-1);
  xStrT.xltype=xltypeStr;
  xStrT.val.str=szBuf;
  xIntT.xltype=xltypeInt;
  xIntT.val.w=2;
  Excel4(xlcAlert,0,2,&xStrT,&xIntT);
  return(1);
  }

__declspec(dllexport) int xlAutoRemove(void)
  {
  char szBuf[255];
  static XLOPER xStr,xInt;
  sprintf(szBuf," %s Removed!\nbuilt on %s at %s",gModuleName+1,__DATE__,__TIME__);
  szBuf[0]=(char)(strlen(szBuf)-1);
  xStr.xltype=xltypeStr;
  xStr.val.str=szBuf;
  xInt.xltype=xltypeInt;
  xInt.val.w=2;
  Excel4(xlcAlert,0,2,&xStr,&xInt);
  return(1);
  }

void CopyPascalString(char*str1,char*str2)
  {
  short i,len;
  len=str1[0];
  for(i=0;i<=len;i++)
    str2[i]=str1[i];
  }

void ConcatenatePascalString(char*str1,char*str2)
  {
  short i,len1,len2;
  len1=str1[0];
  len2=str2[0];
  str1[0]=len1+len2;
  for(i=0;i<len2;i++)
    str1[len1+i]=str2[i+1];
  }

__declspec(dllexport) LPXLOPER xlAddInManagerInfo(LPXLOPER xAction)
  {
  static XLOPER xInfo,xIntAction,xInt;
  static char tempBuf[255];
  xInt.xltype=xltypeInt;
  xInt.val.w=xltypeInt;
  Excel4(xlCoerce,&xIntAction,2,xAction,&xInt);
  if(xIntAction.val.w==1)
   {
   xInfo.xltype=xltypeStr;
   xInfo.val.str=gModuleName;
   }
  else
   {
   xInfo.xltype=xltypeErr;
   xInfo.val.err=xlerrValue;
   }
  return((LPXLOPER) &xInfo);
  }

/*******************************************************************************
  MATHEMATICAL FUNCTIONS
*******************************************************************************/

double*CreateCubicSpline(double*X,double*Y,int n)
  {
  int i,k,l;
  double*C,d,q,t;
  if(n<2)
    return(NULL);
  for(i=1;i<n;i++)
    if(X[i]<=X[i-1])
      return(NULL);
  if((C=calloc(3*n,sizeof(double)))==NULL)
    return(NULL);
  if(n==2)
    {
    C[0]=(Y[1]-Y[0])/(X[1]-X[0]);
    C[1]=0;
    C[2]=0;
    C[3]=C[0];
    C[4]=0;
    C[5]=0;
    return(C);
    }
  d=X[1]-X[0];
  C[2]=d;
  C[4]=(Y[1]-Y[0])/d;
  for(i=1;i<n-1;i++)
    {
    k=3*i;
    t=X[i+1]-X[i];
    C[k+2]=t;
    C[k]=2*(d+t);
    q=(Y[i+1]-Y[i])/t;
    C[k+4]=q;
    C[k+1]=q-C[k+1];
    d=t;
    }
  l=3*(n-1);
  C[0]=-C[2];
  C[l]=-C[l-1];
  C[1]=0;
  C[l+1]=0;
  if(n>3)
    {
    C[1]=C[7]/(X[3]-X[1])-C[4]/(X[2]-X[0]);
    C[l+1]=C[l-2]/(X[n-1]-X[n-3])-C[l-5]/(X[n-2]-X[n-4]);
    C[1]=C[1]*C[2]*C[2]/(X[3]-X[0]);
    C[l+1]=-C[l+1]*C[l-1]*C[l-1]/(X[n-1]-X[n-4]);
    }
  for(i=1;i<n;i++)
    {
    k=3*i;
    t=C[k-1]/C[k-3];
    C[k]-=t*C[k-1];
    C[k+1]-=t*C[k-2];
    }
  C[l+1]=C[l+1]/C[l];
  for(i=n-1;i>0;i--)
    {
    k=3*(i-1);
    C[k+1]=(C[k+1]-C[k+2]*C[k+4])/C[k];
    }
  C[l]=(Y[n-1]-Y[n-2])/C[l-1]+C[l-1]*(C[l-2]+2*C[l+1]);
  for(i=0;i<n-1;i++)
    {
    k=3*i;
    C[k]=(Y[i+1]-Y[i])/C[k+2]-C[k+2]*(C[k+4]+2*C[k+1]);
    C[k+2]=(C[k+4]-C[k+1])/C[k+2];
    C[k+1]*=3;
    }
  C[l+1]*=3;
  C[l+2]=C[l-1];
  return(C);
  }

double EvaluateCubicSpline(double*X,double*Y,int n,double*C,double x,int m)
  {
  int i,i1,i2,k,n1;
  if(n<2)
    return(0);
  n1=n-1;
  if(x<=X[0])
    i=0;
  else if(x>=X[n1])
    i=n1;
  else
    {
    i1=0;
    i2=n1;
    for(k=0;k<32;k++)
      {
      i=(i1+i2)/2;
      if(x<X[i]||i==n1)
        i2=i;
      else if(x>X[i+1])
        i1=i;
      else
        break;
      }
    }
  x-=X[i];
  k=3*i;
  if(m==1)
    return((3*C[k+2]*x+2*C[k+1])*x+C[k]);
  if(m==2)
    return(6*C[k+2]*x+2*C[k+1]);
  if(m==3)
    return(6*C[k+2]);
  return(((C[k+2]*x+C[k+1])*x+C[k])*x+Y[i]);
  }

int SimultaneousEquations(double*A,double*B,int n)
  {
  int i,j,k,ip,jp,*pivot;
  double a,p,b;
  if(n<1)
    return(__LINE__);
  if(n==1)
    {
    a=A[0];
    if(fabs(a)<FLT_MIN)
      return(__LINE__);
    B[0]/=a;
    return(0);
    }
  if((pivot=calloc(n,sizeof(int)))==NULL)
    return(__LINE__);
  for(i=0;i<n;i++)
    pivot[i]=i;
  for(k=0;k<n-1;k++)
    {
    ip=k;
    jp=k;
    p=fabs(A[k*(n+1)]);
    for(i=k;i<n;i++)
      {
      for(j=k;j<n;j++)
        {
        a=fabs(A[n*i+j]);
        if(a>p)
          {
          ip=i;
          jp=j;
          p=a;
          }
        }
      }
    if(p<FLT_MIN)
      {
      free(pivot);
      return(__LINE__);
      }
    if(ip!=k)
      {
      b=B[k];
      B[k]=B[ip];
      B[ip]=b;
      for(j=k;j<n;j++)
        {
        a=A[n*k+j];
        A[n*k+j]=A[n*ip+j];
        A[n*ip+j]=a;
        }
      }
    if(jp!=k)
      {
      j=pivot[jp];
      pivot[jp]=pivot[k];
      pivot[k]=j;
      for(i=0;i<n;i++)
        {
        a=A[n*i+k];
        A[n*i+k]=A[n*i+jp];
        A[n*i+jp]=a;
        }
      }
    for(i=k+1;i<n;i++)
      {
      a=A[n*i+k]/A[n*k+k];
      B[i]-=a*B[k];
      for(j=k+1;j<n;j++)
        A[n*i+j]-=a*A[n*k+j];
      }
    }
  a=A[n*n-1];
  p=fabs(a);
  if(p<FLT_MIN)
    {
    free(pivot);
    return(__LINE__);
    }
  B[n-1]/=a;
  for(k=1;k<n;k++)
    {
    i=n-1-k;
    b=0.;
    for(j=i+1;j<n;j++)
      b+=A[n*i+j]*B[j];
    B[i]=(B[i]-b)/A[n*i+i];
    }
  for(i=0;i<n;i++)
    A[i]=B[i];
  for(i=0;i<n;i++)
    B[pivot[i]]=A[i];
  free(pivot);
  return(0);
  }

int LinearLeastSquares(double*X,double*Y,int m,double*a,int n)
  {
  int i,j,k;
  double*ata,*atb;
  if(n<2)
    return(__LINE__);
  if(m<n)
    return(__LINE__);
  if((atb=calloc(n,sizeof(double)))==NULL)
    return(__LINE__);
  if((ata=calloc(n*n,sizeof(double)))==NULL)
    {
    free(atb);
    return(__LINE__);
    }
  for(k=0;k<m;k++)
    {
    a[0]=1.;
    for(i=1;i<n;i++)
      a[i]=a[i-1]*X[k];
    for(i=0;i<n;i++)
      {
      atb[i]+=a[i]*Y[k];
      for(j=0;j<n;j++)
        ata[n*i+j]+=a[i]*a[j];
      }
    }
  if(SimultaneousEquations(ata,atb,n))
    {
    free(atb);
    free(ata);
    return(__LINE__);
    }
  for(i=0;i<n;i++)
    a[i]=atb[i];
  free(atb);
  free(ata);
  return(0);
  }

/* This is how linear least-squares with linear constraints works...

   The least-squares part is A(m,n)*X(n,1)=B(n,1), where m>=n.
   Here X(1,k)=1, X(2,k)=Xr(k), X(3,k)=Xr(k)^2, X(4,k)=Xr(k)^3, ... B(k)=Yr(k), for k=1 to m

   The linear constraint part is D(l,m)*X(m,1)=E(l,m), where l>=1.
   Here X(1,k)=1, X(2,k)=Xc(k), X(3,k)=Xc(k)^2, X(4,k)=Xc(k)^3, ... E(k)=Yc(k), for k=1 to l

   The complete matrix that's solved looks like:

     --------   ---   -----
     |AtA|Dt|   |C|   |AtB|
     -------- * --- = -----
     | D |0 |   |L|   | E |
     --------   ---   -----

     where: A[m,n] (the X data expanded as polynomials)
            B[m,1] (the Y data)
            D[l,n] (the X constraints expanded as polynomials)
            E[l,1] (the Y constraints)
            AtA[n,n] is Atranspose[n,m] times A[m,n]
            AtB[n,1] is Atranspose[n,m] times B[m,1]
            Dt[n,l] is Dtranspose
            C[n,1] are the coefficients
            L[l,1] are the Lagrange multipliers (you don't
              actually use these for anything in this case)
*/

int LinearLeastSquaresWithLinearConstraints(double*Xr,double*Yr,int nr,int nt,double*Xc,double*Yc,int nc,double*a)
  {
  int i,j,k,l,n;
  double*ata,*atb;

/* check for errors */

  if(nt<2)
    return(__LINE__);
  if(nc<1)
    return(__LINE__);
  n=nt+nc;
  if(nr<n)
    return(__LINE__);

/* allocate arrays */

  if((atb=calloc(n,sizeof(double)))==NULL)
    return(__LINE__);
  if((ata=calloc(n*n,sizeof(double)))==NULL)
    {
    free(atb);
    return(__LINE__);
    }

/* add least-squares part */

  for(k=0;k<nr;k++)
    {
    a[0]=1.;
    for(i=1;i<nt;i++)
      a[i]=a[i-1]*Xr[k];
    for(i=0;i<nt;i++)
      {
      atb[i]+=a[i]*Yr[k];
      for(j=0;j<nt;j++)
        ata[n*i+j]+=a[i]*a[j];
      }
    }

/* add constraints part */

  for(l=0;l<nc;l++)
    {
    a[0]=1.;
    for(i=1;i<nt;i++)
      a[i]=a[i-1]*Xc[l];
    atb[nt+l]=Yc[l];
    for(i=0;i<nt;i++)
      {
      ata[n*(nt+l)+i]=a[i];
      ata[n*i+nt+l]=a[i];
      }
    }

/* solve simultaneous equations */

  if(SimultaneousEquations(ata,atb,n))
    {
    free(atb);
    free(ata);
    return(__LINE__);
    }
  for(i=0;i<nt;i++)
    a[i]=atb[i];
  free(atb);
  free(ata);
  return(0);
  }

/*******************************************************************************
  RETURN AND ARRAY PASSING FUNCTIONS
*******************************************************************************/

XLOPER result;

LPXLOPER xlError()
  {
  result.xltype=xltypeErr;
  result.val.err=999;
  return(&result);
  }

LPXLOPER xlResult(double r)
  {
  result.xltype=xltypeNum;
  result.val.num=r;
  return(&result);
  }

#define MAX_VALS 20
struct{
  XLOPER Xlop[MAX_VALS];
  double Xdbl[MAX_VALS];
  }Vals;

LPXLOPER xlArray(int rows,int cols)
  {
  int i;
  for(i=0;i<rows*cols;i++)
    {
    Vals.Xlop[i].xltype=xltypeNum;
    Vals.Xlop[i].val.num=Vals.Xdbl[i];
    }
  result.xltype=xltypeMulti;
  result.val.array.rows=rows;
  result.val.array.columns=cols;
  result.val.array.lparray=Vals.Xlop;
  return(&result);
  }

typedef struct{int n,m,cols,rows;double*v;}XBUF;
struct{XBUF x,y,z,u,v;}xbuf;

int GetArray(LPXLOPER xArray,XBUF*xBuf,int ignore0)
  {
  int hasXMulti=0,hasTargMulti=0,error=-1;
  int i;
  XLOPER xMulti,*xPtr,tempTypeMulti;
  tempTypeMulti.xltype=xltypeInt;
  tempTypeMulti.val.w=xltypeMulti;
  if(xArray->xltype!=xltypeRef&&xArray->xltype!=xltypeSRef&&xArray->xltype!=xltypeMulti)
    return(-__LINE__);
  if(xlretUncalced==Excel4(xlCoerce,&xMulti,2,xArray,&tempTypeMulti))
    return(-__LINE__);
  xBuf->rows=xMulti.val.array.rows;
  xBuf->cols=xMulti.val.array.columns;
  xBuf->n=xMulti.val.array.columns*xMulti.val.array.rows;
  if(xBuf->n<1)
    {
    Excel4(xlFree,0,1,(LPXLOPER)&xMulti);
    return(-__LINE__);
    }
  if(xBuf->m==0)
    {
    xBuf->m=max(1024,xBuf->n);
    xBuf->v=calloc(xBuf->m,sizeof(double));
    }
  else if(xBuf->n>xBuf->m)
    {
    free(xBuf->v);
    xBuf->m=max(xBuf->m*2,xBuf->n);
    xBuf->v=calloc(xBuf->m,sizeof(double));
    }
  xPtr=xMulti.val.array.lparray;
  for(i=0;i<xBuf->n;i++)
    {
    if(i==0&&ignore0)
      xBuf->v[i]=0.;
    else if(xPtr[i].xltype!=xltypeNum)
      {
      if(xPtr[i].xltype==xltypeErr)
        error=xPtr[i].val.err;
      else
        error=xlerrValue;
      goto done;
      }
    else
      xBuf->v[i]=xPtr[i].val.num;
    }
done:
  Excel4(xlFree,0,1,(LPXLOPER)&xMulti);
  if(error!=-1)
    return(-__LINE__);
  return(xBuf->n);
  }

/*******************************************************************************
  EXCEL FUNCTIONS
*******************************************************************************/

__declspec (dllexport) LPXLOPER Lagrange(LPXLOPER xArray,LPXLOPER yArray,double x)
  {
  int i,j;
  double r,R;
  if(GetArray(xArray,&xbuf.x,0)<2)
    return(xlError());
  if(GetArray(yArray,&xbuf.y,0)<2)
    return(xlError());
  if(xbuf.x.n!=xbuf.y.n)
    return(xlError());
  R=0.;
  for(i=0;i<xbuf.x.n;i++)
    {
    r=xbuf.y.v[i];
    for(j=0;j<xbuf.x.n;j++)
      if(j!=i)
        r*=(x-xbuf.x.v[j])/(xbuf.x.v[i]-xbuf.x.v[j]);
    R+=r;
    }
  return(xlResult(R));
  }

__declspec (dllexport) LPXLOPER Spline(LPXLOPER xArray,LPXLOPER yArray,double x)
  {
  int i;
  double*C,*X,*Y,y;
  if(GetArray(xArray,&xbuf.x,0)<2)
    return(xlError());
  if(GetArray(yArray,&xbuf.y,0)<2)
    return(xlError());
  if(xbuf.x.n!=xbuf.y.n)
    return(xlError());
  if(xbuf.x.n<4)
    return(xlError());
  if((X=calloc(2*xbuf.x.n,sizeof(double)))==NULL)
    return(xlError());
  Y=X+xbuf.x.n;
  for(i=0;i<xbuf.x.n;i++)
    {
    X[i]=xbuf.x.v[i];
    Y[i]=xbuf.y.v[i];
    }
  C=CreateCubicSpline(X,Y,xbuf.x.n);
  if(C==NULL)
    {
    free(X);
    return(xlError());
    }
  y=EvaluateCubicSpline(X,Y,xbuf.x.n,C,x,0);
  free(X);
  free(C);
  return(xlResult(y));
  }

__declspec (dllexport) LPXLOPER LLSQ(LPXLOPER xArray,LPXLOPER yArray,_int32 terms)
  {
  if(terms<2||terms>MAX_VALS)
    return(xlError());
  if(GetArray(xArray,&xbuf.x,0)<2)
    return(xlError());
  if(GetArray(yArray,&xbuf.y,0)<2)
    return(xlError());
  if(xbuf.x.n!=xbuf.y.n)
    return(xlError());
  if(xbuf.x.n<terms)
    return(xlError());
  if(LinearLeastSquares(xbuf.x.v,xbuf.y.v,xbuf.x.n,Vals.Xdbl,terms))
    return(xlError());
  return(xlArray(1,terms));
  }

__declspec (dllexport) LPXLOPER LLSQLC(LPXLOPER xArray,LPXLOPER yArray,_int32 terms,LPXLOPER uArray,LPXLOPER vArray)
  {
  if(terms<2||terms>MAX_VALS)
    return(xlError());
  if(GetArray(xArray,&xbuf.x,0)<2)
    return(xlError());
  if(GetArray(yArray,&xbuf.y,0)<2)
    return(xlError());
  if(xbuf.x.n!=xbuf.y.n)
    return(xlError());
  if(xbuf.x.n<terms)
    return(xlError());
  if(GetArray(uArray,&xbuf.u,0)<1)
    return(xlError());
  if(GetArray(vArray,&xbuf.v,0)<1)
    return(xlError());
  if(xbuf.u.n!=xbuf.v.n)
    return(xlError());
  if(LinearLeastSquaresWithLinearConstraints(xbuf.x.v,xbuf.y.v,xbuf.x.n,terms,xbuf.u.v,xbuf.v.v,xbuf.u.n,Vals.Xdbl))
    return(xlError());
  return(xlArray(1,terms));
  }
