#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

void SlowFourierTransform(double*f,int n,double*c,double*s,int m)
  {
  int i,j;
  double ci,cj,cs,si,sj,ss;
  for(i=0;i<m;i++)
    {
    for(cj=cs=sj=ss=j=0;j<n;j++)
      {
      ci=cos(2*i*(j+1)*M_PI/n);
      si=sin(2*i*(j+1)*M_PI/n);
      cj+=f[j]*ci;
      sj+=f[j]*si;
      cs+=ci*ci;
      ss+=si*si;
      }
    c[i]=cj/cs;
    if(i)
      s[i]=sj/ss;
    }
  }

void FastFourierTransform(double*f,int n,double*c,double*s,int m)
  {
  int i,j;
  double cc,ci,cj,ck,si,sj,sk,ss;
  for(i=0;i<m;i++)
    {
    ci=cj=cos(2*i*M_PI/n);
    si=sj=sin(2*i*M_PI/n);
    for(c[i]=s[i]=cc=ss=j=0;j<n;j++)
      {
      c[i]+=cj*f[j];
      s[i]+=sj*f[j];
      cc+=cj*cj;
      ss+=sj*sj;
      ck=ci*cj-si*sj;
      sk=si*cj+ci*sj;
      cj=ck;
      sj=sk;
      }
    c[i]/=cc;
    if(i)
      s[i]/=ss;
    }
  }

void SquareWave(double*f,int n)
  {
  int j;
  printf("              Square Wave\n");
  for(j=0;j<n;j++)
    {
    if(j<n/2)
      f[j]=M_PI/4.;
    else if(j>n/2)
      f[j]=-M_PI/4.;
    else
      f[j]=0.;
    }
  }

void SawTooth(double*f,int n)
  {
  int j;
  printf("               Saw Tooth\n");
  for(j=0;j<n;j++)
    {
    if(j<n/2)
      f[j]=j*M_PI/n;
    else if(j>n/2)
      f[j]=(j+1-n)*M_PI/n;
    else
      f[j]=0.;
    }
  }

void Ramp(double*f,int n)
  {
  int j;
  double r;
  printf("                  Ramp\n");
  for(j=0;j<n;j++)
    {
    if(j<n/2)
      r=j/(n/2.);
    else if(j>n/2)
      r=(n-1-j)/(n/2.);
    else
      r=1.;
    f[j]=(0.5-r)*M_PI*M_PI/4.;
    }
  }

#define n 5000
double f[n];

#define m 100
double c1[m],c2[m];
double s1[m],s2[m];

extern int __stdcall QueryPerformanceFrequency(__int64*);
extern int __stdcall QueryPerformanceCounter(__int64*);

void Example(void func(double*,int))
  {
  int i;
  __int64 rate,t1,t2,t3;
  QueryPerformanceFrequency(&rate);
  func(f,n);
  QueryPerformanceCounter(&t1);
  SlowFourierTransform(f,n,c1,s1,m);
  QueryPerformanceCounter(&t2);
  FastFourierTransform(f,n,c2,s2,m);
  QueryPerformanceCounter(&t3);
  printf("   <-----slow----->  <-----fast----->\n");
  printf("i    c[i]     s[i]     c[i]     s[i]\n");
  for(i=0;i<min(10,m);i++)
    printf("%i %8.5lf %8.5lf %8.5lf %8.5lf\n",i,c1[i],s1[i],c2[i],s2[i],c2[i]-c1[i],s2[i]-s1[i]);
  printf("  %8.5lf  seconds %8.5lf  seconds\n",(t2-t1)/((double)rate),(t3-t2)/((double)rate));
  }

int main(int argc,char**argv,char**envp)
  {
  Example(SquareWave);
  Example(SawTooth);
  Example(Ramp);
  return(0);
  }
