/* insertion sort examples by Dudley J. Benton */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

struct{int n,m;double*d;}list;

void InsertPoint(double p)
  {
  int i;
  if(list.m==0)
    {
    list.m=10;
    if((list.d=calloc(list.m,sizeof(double)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    list.d[0]=p;
    list.n=1;
    return;
    }
  for(i=0;i<list.n;i++)
    {
    if(list.d[i]<p)
      continue;
    if(list.d[i]>p)
      break;
    return;
    }
  if(list.n>=list.m)
    {
    double*t=list.d;
    list.m+=10;
    if((list.d=calloc(list.m,sizeof(double)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    memcpy(list.d,t,list.n*sizeof(double));
    free(t);
    }
  if(i<list.n)
    memmove(list.d+i+1,list.d+i,(list.n-i)*sizeof(double));
  list.d[i]=p;
  list.n++;
  }

void InsertPoints(double*p,int n)
  {
  int i;
  printf("inserting %i points\n",n);
  memset(&list,0,sizeof(list));
  for(i=0;i<n;i++)
    InsertPoint(p[i]);
  printf("%i unique points\n",list.n);
  for(i=0;i<list.n;i++)
    printf("%7.4lf\n",list.d[i]);
  }

int compare(double p,double q)
  {
  if(p<q)
    return(-1);
  if(p>q)
    return(1);
  return(0);
  }

void BisectPoint(double p)
  {
  int i,j,j1,j2,k;
  if(list.m==0)
    {
    list.m=10;
    if((list.d=calloc(list.m,sizeof(double)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    list.d[0]=p;
    list.n=1;
    return;
    }
  j1=0;
  j2=list.n;
  for(i=0;i<32;i++)
    {
    j=(j1+j2)/2;
    k=compare(p,list.d[j]);
    if(k==0)
      return;
    if(j1==j)
      {
      if(k>0)
        j++;
      break;
      }
    else if(k>0)
      j1=j;
    else
      j2=j;
    }
  if(list.n>=list.m)
    {
    double*t=list.d;
    list.m+=10;
    if((list.d=calloc(list.m,sizeof(double)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    memcpy(list.d,t,list.n*sizeof(double));
    free(t);
    }
  if(j<list.n)
    memmove(list.d+j+1,list.d+j,(list.n-j)*sizeof(double));
  list.d[j]=p;
  list.n++;
  }

void BisectPoints(double*p,int n)
  {
  int i;
  printf("inserting %i points with bisection search\n",n);
  memset(&list,0,sizeof(list));
  for(i=0;i<n;i++)
    BisectPoint(p[i]);
  printf("%i unique points\n",list.n);
  for(i=0;i<list.n;i++)
    printf("%7.4lf\n",list.d[i]);
  }

typedef struct{double x,y,z;}NODE;
struct{int*l,m,n;NODE*node;}heap;

int CompareNodes(NODE p,NODE q)
  {
  if(p.x<q.x)
    return(-1);
  if(p.x>q.x)
    return(1);
  if(p.y<q.y)
    return(-1);
  if(p.y>q.y)
    return(1);
  if(p.z<q.z)
    return(-1);
  if(p.z>q.z)
    return(1);
  return(0);
  }

void BisectNode(NODE p)
  {
  int i,j,j1,j2,k;
  if(heap.m==0)
    {
    heap.m=10;
    if((heap.l=calloc(heap.m,sizeof(int)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    if((heap.node=calloc(heap.m,sizeof(NODE)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    heap.node[0]=p;
    heap.l[0]=0;
    heap.n=1;
    return;
    }
  j1=0;
  j2=heap.n;
  for(i=0;i<32;i++)
    {
    j=(j1+j2)/2;
    k=CompareNodes(p,heap.node[heap.l[j]]);
    if(k==0)
      return;
    if(j1==j)
      {
      if(k>0)
        j++;
      break;
      }
    else if(k>0)
      j1=j;
    else
      j2=j;
    }
  if(heap.n>=heap.m)
    {
    int*l=heap.l;
    NODE*t=heap.node;
    heap.m+=10;
    if((heap.l=calloc(heap.m,sizeof(int)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    memcpy(heap.l,l,heap.n*sizeof(int));
    free(l);
    if((heap.node=calloc(heap.m,sizeof(NODE)))==NULL)
      {
      printf("can't allocate memory for points\n");
      exit(1);
      }
    memcpy(heap.node,t,heap.n*sizeof(NODE));
    free(t);
    }
  if(j<heap.n)
    memmove(heap.l+j+1,heap.l+j,(heap.n-j)*sizeof(int));
  heap.l[j]=heap.n;
  heap.node[heap.n]=p;
  heap.n++;
  }

void BisectNodes(double*p,int n)
  {
  int i,j,m=1000;
  NODE node;
  printf("inserting %i nodes with bisection search\n",m);
  memset(&heap,0,sizeof(heap));
  for(i=0;i<m;i++)
    {
    node.x=p[rand()%n];
    node.y=p[rand()%n];
    node.z=p[rand()%n];
    BisectNode(node);
    }
  printf("%i unique nodes\n",heap.n);
  for(i=0;i<heap.n;i++)
    {
    j=heap.l[i];
    printf("%7.4lf %7.4lf %7.4lf\n",heap.node[j].x,heap.node[j].y,heap.node[j].z);
    }
  }

double points[]={ 0.0327, 0.0287, 0.0625, 0.7196, 0.7240,-0.0287,-0.0327,
  0.0315, 0.7218, 0.7164, 0.0685, 0.0465, 0.7167, 0.0364, 0.0369, 0.7335,
  0.0295, 0.0300,-0.0300, 0.7182, 0.0345, 0.0322, 0.0615, 0.0615, 0.7251,
  0.7218, 0.7284, 0.0615, 0.7262, 0.0515, 0.0615, 0.7251, 0.0615, 0.0373,
  0.0318, 0.7182, 0.0615,-0.0293, 0.7305, 0.0618,-0.0284,-0.0311,-0.1045};

int main(int argc,char**argv,char**envp)
  {
//InsertPoints(points,sizeof(points)/sizeof(double));
//BisectPoints(points,sizeof(points)/sizeof(double));
  BisectNodes(points,sizeof(points)/sizeof(double));
  return(0);
  }
