#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <float.h>
#include <math.h>

int SimultaneousLinearEquations(double*A,double*B,int n)
  {
  int i,j,k,ip,jp,*pivot;
  double a,b,*C,p;
  if(n<1)
    return(__LINE__);
  if(n==1)
    {
    a=A[0];
    if(fabs(a)<FLT_MIN)
      return(__LINE__);
    B[0]/=a;
    return(0);
    }
  pivot=calloc(n,sizeof(int));
  for(i=0;i<n;i++)
    pivot[i]=i;
  for(k=0;k<n-1;k++)
    {
    ip=k;
    jp=k;
    p=fabs(A[k*(n+1)]);
    for(i=k;i<n;i++)
      {
      for(j=k;j<n;j++)
        {
        a=fabs(A[n*i+j]);
        if(a>p)
          {
          ip=i;
          jp=j;
          p=a;
          }
        }
      }
    if(p<FLT_MIN)
      {
      free(pivot);
      return(__LINE__);
      }
    if(ip!=k)
      {
      b=B[k];
      B[k]=B[ip];
      B[ip]=b;
      for(j=k;j<n;j++)
        {
        a=A[n*k+j];
        A[n*k+j]=A[n*ip+j];
        A[n*ip+j]=a;
        }
      }
    if(jp!=k)
      {
      j=pivot[jp];
      pivot[jp]=pivot[k];
      pivot[k]=j;
      for(i=0;i<n;i++)
        {
        a=A[n*i+k];
        A[n*i+k]=A[n*i+jp];
        A[n*i+jp]=a;
        }
      }
    for(i=k+1;i<n;i++)
      {
      a=A[n*i+k]/A[n*k+k];
      B[i]-=a*B[k];
      for(j=k+1;j<n;j++)
        A[n*i+j]-=a*A[n*k+j];
      }
    }
  a=A[n*n-1];
  p=fabs(a);
  if(p<FLT_MIN)
    {
    free(pivot);
    return(__LINE__);
    }
  C=A+n*(n-1);
  C[n-1]=B[n-1]/a;
  for(k=1;k<n;k++)
    {
    i=n-1-k;
    b=0;
    for(j=i+1;j<n;j++)
      b+=A[n*i+j]*C[j];
    C[i]=(B[i]-b)/A[n*i+i];
    }
  for(i=0;i<n;i++)
    B[pivot[i]]=C[i];
  free(pivot);
  return(0);
  }

double*PolynomialRegression(double*X,double*Y,int n,int m)
  {
  int i,j,k;
  double*A,*AtA,*AtB;
  A=calloc(n,sizeof(double));
  AtB=calloc(n,sizeof(double));
  AtA=calloc(n*n,sizeof(double));
  for(k=0;k<m;k++)
    {
    A[0]=1.;
    for(i=1;i<n;i++)
      A[i]=A[i-1]*X[k];
    for(i=0;i<n;i++)
      {
      AtB[i]+=A[i]*Y[k];
      for(j=0;j<n;j++)
        AtA[n*i+j]+=A[i]*A[j];
      }
    }
  i=SimultaneousLinearEquations(AtA,AtB,n);
  free(A);
  free(AtA);
  if(i==0)
    return(AtB);
  free(AtB);
  return(NULL);
  }

double Polynomial(double*C,int n,double X)
  {
  int i;
  double P;
  P=C[n-1];
  for(i=n-2;i>=0;i--)
    P=P*X+C[i];
  return(P);
  }

struct{double x,y;}data[]={
  {-5.98859966 ,0.490214706},
  {-4.55538666 ,0.193805814},
  {-3.39578187 ,1.81590142 },
  {-2.23617709 ,1.2        },
  {-1.40718222 ,3.75165544 },
  {-0.703393502,3.72       },
  { 0.065      ,5.217348   },
  { 0.79299829 ,4.15       },
  { 1.99278358 ,4.42900431 },
  { 3.48619419 ,5.04       },
  { 4.38761163 ,1.90541104 },
  { 5.31920958 ,2.04       },
  { 6.02280068 ,0.27       },
  { 7.61186586 ,0.358193967}};

#define m (sizeof(data)/sizeof(data[0]))
#define n 5

int mm[m],nn[m];
double E[m],X[n],Y[n];

double Error(int silent)
  {
  int i;
  double*C,Emax;
  for(i=0;i<n;i++)
    {
    X[i]=data[mm[i]].x;
    Y[i]=data[mm[i]].y;
    }
  C=PolynomialRegression(X,Y,n,n);
  Emax=0.;
  for(i=0;i<m;i++)
    {
    E[i]=fabs(Polynomial(C,n,data[i].x)-data[i].y);
    if(!silent)
      printf("%lf\t%lf\t%lf\t%lf\n",data[i].x,data[i].y,Polynomial(C,n,data[i].x),E[i]);
    if(E[i]>Emax)
      Emax=E[i];
    }
  if(!silent)
    for(i=0;i<n;i++)
      printf("%.12lE\n",C[i]);
  free(C);
  for(i=0;i<m;i++)
    printf("%2i ",mm[i]);
  printf("%lf\n",Emax);
  return(Emax);
  }

int CompareErrors(const void*v1,const void*v2)
  {
  int i,j;
  i=*(int*)v1;
  j=*(int*)v2;
  if(E[i]<E[j])
    return(1);
  if(E[i]>E[j])
    return(-1);
  return(0);
  }

int largest(int k)
  {
  int i;
  for(i=0;i<m;i++)
    nn[i]=i;
  qsort(nn,m,sizeof(int),CompareErrors);
  for(i=0;i<m;i++)
    printf("%i %i %lf\n",i,nn[i],E[nn[i]]);
  for(i=0;i<m;i++)
    if(mm[i]==nn[k])
      return(i);
  return(-1);
  }

int main(int argc,char**argv,char**envp)
  {
  int i,j,k,kk,l;
  double e,ex,exx;
  for(i=0;i<m;i++)
    mm[i]=i;
  exx=DBL_MAX;
  for(k=0;k<m-n;)
    {
    Error(1);
    l=largest(k);
    printf("largest=%i\n",mm[l]);
    kk=-1;
    ex=DBL_MAX;
    for(i=0;i<n;i++)
      {
      j=mm[i];
      mm[i]=mm[l];
      mm[l]=j;
      e=Error(1);
      if(e<ex)
        {
        ex=e;
        kk=i;
        }
      mm[l]=mm[i];
      mm[i]=j;
      }
    if(kk<0||exx<ex)
      {
      printf("no improvement\n");
      k++;
      continue;
      }
    j=mm[kk];
    mm[kk]=mm[l];
    mm[l]=j;
    exx=Error(1);
    printf("best so far\n");
    }
  Error(0);
  return(0);
  }
  