#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#define _USE_MATH_DEFINES
#include <math.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

double erf(double x)
  {
  double q,t,y,z;
  if(fabs(x)<0.0000001)
    return(0.);
  if(x>6.)
    return(1.);
  if(x<-6.)
    return(-1.);
  y=fabs(x);
  t=1./(1.+0.3275911*y);
  q=((((1.061405429*t-1.453152027)*t+1.421413741)*t
    -0.284496736)*t+0.254829592)*t;
  z=1.-q/exp(x*x);
  if(x<0.)
    return(-z);
  return(z);
  }

double erfc(double x)
  {
  return(1.-erf(x));
  }

double normdist(double x,double mu,double sigma,BOOL cumulative)
  {
  double z;
  z=(x-mu)/sigma;
  if(cumulative)
    return((1.+erf(z/sqrt(2.)))/2.);
  return(exp(-z*z/2.)/sigma/sqrt(2.*M_PI));
  }

double normsdist(double z)
  {
  double x;
  x=(((0.00491269829888*z*z+0.226256306341)*z*z+1.31706921487)*z*z
    +0.87779133934)*z/(((0.000456826821739*z*z+0.0860790688367)*z*z
    +1.02941402653)*z*z+1.);
  if(z<0.)
    return(exp(x)/2.);
  if(z>0.)
    return(1.-exp(-x)/2.);
  return(0.5);
  }

double normsinv(double p)
  {
  double x;
  if(p<0.5)
    x=log(2.*p);
  else if(p>0.5)
    x=-log(2.-2.*p);
  else
    x=0.;
  return(((((((((-3.25827445083E-9*x+1.61986324187E-15)*x
    +0.000018303435409)*x-6.2769338749E-12)*x+0.00758688240346)*x
    -1.43922019269E-9)*x+0.306055996906)*x-1.92368212218E-8)*x
    +1.02666429167)*x/((((((7.81461220935E-5*x-2.38842480642E-11)*x
    +0.0181678641217)*x-2.89307816431E-9)*x+0.468538918156)*x
    -2.18209886841E-8)*x+1));
  }

double norminv(double p,double mu,double sigma)
  {
  return(mu+normsinv(p)*sigma);
  }

int main(int argc,char**argv,char**envp)
  {
  double mu=100.,p1,p2,p3,sigma=15.,x1,x2,z1,z2;
  printf("              FALSE      TRUE\n");
  printf(" x     z    NORMDIST   NORMDIST   NORMSDIST    x      z\n");
  for(x1=20.;x1<181.;x1+=10.)
    {
    z1=(x1-mu)/sigma;
    p1=normdist(x1,mu,sigma,FALSE);
    p2=normdist(x1,mu,sigma,TRUE);
    p3=normsdist(z1);
    x2=norminv(p3,mu,sigma);
    z2=normsinv(p3);
    printf("%3.0lf %6.3lf %10.8lf %10.8lf %10.8lf %6.2lf %6.3lf\n",x1,z1,p1,p2,p3,x2,z2);
    }
  return(0);
  }
