/* qsort examples by D. James Benton */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "qsort.h"

int CompareStrings(const void*a,const void*b)
  {
  return(_stricmp(*(char**)a,*(char**)b));
  }

typedef struct{char*name,*country;int population;}CITY;

/* World's 87 Largest Cities by Population */

CITY cities[]={
  {"Chongqing"       ,"China"        ,28846170},
  {"Shanghai"        ,"China"        ,23019148},
  {"Beijing"         ,"China"        ,19612368},
  {"Karachi"         ,"Pakistan"     ,18000000},
  {"Tokyo"           ,"Japan"        ,13161751},
  {"Tianjin"         ,"China"        ,12938224},
  {"Guangzhou"       ,"China"        ,12700800},
  {"Mumbai"          ,"India"        ,12478447},
  {"Moscow"          ,"Russia"       ,11514330},
  {"Istanbul"        ,"Turkey"       ,11372613},
  {"Sao Paulo"       ,"Brazil"       ,11244369},
  {"Delhi"           ,"India"        ,11007835},
  {"Chengdu"         ,"China"        ,10820000},
  {"Seoul"           ,"South Korea"  ,10464051},
  {"Shenzhen"        ,"China"        ,10357938},
  {"Harbin"          ,"China"         ,9874000},
  {"Wuhan"           ,"China"         ,9785392},
  {"Jakarta"         ,"Indonesia"     ,9588198},
  {"Shijiazhuang"    ,"China"         ,9170000},
  {"Wenzhou"         ,"China"         ,9122100},
  {"Mexico City"     ,"Mexico"        ,8841916},
  {"Kinshasa"        ,"Congo"         ,8754000},
  {"Hangzhou"        ,"China"         ,8700400},
  {"Qingdao"         ,"China"         ,8456100},
  {"Bangalore"       ,"India"         ,8425970},
  {"Dongguan"        ,"China"         ,8220237},
  {"New York City"   ,"United States" ,8175133},
  {"London"          ,"United Kingdom",7619800},
  {"Lima"            ,"Peru"          ,7605742},
  {"Ningbo"          ,"China"         ,7605700},
  {"Changchun"       ,"China"         ,7512000},
  {"Xi'an"           ,"China"         ,7410000},
  {"Nanjing"         ,"China"         ,7400000},
  {"Bogota"          ,"Colombia"      ,7259597},
  {"Tehran"          ,"Iran"          ,7241000},
  {"Shenyang"        ,"China"         ,7200000},
  {"Foshan"          ,"China"         ,7194311},
  {"Ho Chi Minh City","Vietnam"       ,7123340},
  {"Shenyang"        ,"China"         ,7100000},
  {"Hong Kong"       ,"Hong Kong"     ,7097600},
  {"Bangkok"         ,"Thailand"      ,7025000},
  {"Dhaka"           ,"Bangladesh"    ,7000940},
  {"Zhanjiang"       ,"China"         ,6993304},
  {"Hyderabad"       ,"India"         ,6809970},
  {"Cairo"           ,"Egypt"         ,6758581},
  {"Hanoi"           ,"Vietnam"       ,6448837},
  {"Rio de Janeiro"  ,"Brazil"        ,6323037},
  {"Lahore"          ,"Pakistan"      ,6318745},
  {"Chennai"         ,"India"         ,6221782},
  {"Dalian"          ,"China"         ,6130000},
  {"Jinan"           ,"China"         ,6039900},
  {"Taizhou"         ,"China"         ,5968800},
  {"Jieyang"         ,"China"         ,5877025},
  {"Maoming"         ,"China"         ,5817753},
  {"Ahmedabad"       ,"India"         ,5570585},
  {"Baghdad"         ,"Iraq"          ,5402486},
  {"Shantou"         ,"China"         ,5391028},
  {"Jinhua"          ,"China"         ,5361600},
  {"Singapore"       ,"Singapore"     ,4987600},
  {"Shaoxing"        ,"China"         ,4912200},
  {"St. Petersburg"  ,"Russia"        ,4848700},
  {"Riyadh"          ,"Saudi Arabia"  ,4848000},
  {"Huizhou"         ,"China"         ,4597002},
  {"Surat"           ,"India"         ,4523022},
  {"Jiaxing"         ,"China"         ,4501700},
  {"Kolkata"         ,"India"         ,4486679},
  {"Jiangmen"        ,"China"         ,4448871},
  {"Yangon"          ,"Myanmar"       ,4350000},
  {"Meizhou"         ,"China"         ,4240139},
  {"Alexandria"      ,"Egypt"         ,4110015},
  {"Ankara"          ,"Turkey"        ,3901201},
  {"Johannesburg"    ,"South Africa"  ,3888180},
  {"Los Angeles"     ,"United States" ,3831868},
  {"Pune"            ,"India"         ,3760636},
  {"Yokohama"        ,"Japan"         ,3689603},
  {"Abidjan"         ,"Cote d'Ivoire" ,3660682},
  {"Busan"           ,"South Korea"   ,3574340},
  {"Nairobi"         ,"Kenya"         ,3500000},
  {"Cape Town"       ,"South Africa"  ,3497097},
  {"Durban"          ,"South Africa"  ,3468086},
  {"Berlin"          ,"Germany"       ,3450900},
  {"Pyongyang"       ,"North Korea"   ,3255388},
  {"Jeddah"          ,"Saudi Arabia"  ,3234000},
  {"Kanpur"          ,"India"         ,3221435},
  {"Madrid"          ,"Spain"         ,3213271},
  {"Jaipur"          ,"India"         ,3210570},
  {"Buenos Aires"    ,"Argentina"     ,3050728},
  {NULL              ,NULL            ,      0}};

void JustNames()
  {
  char**list;
  int n;
  for(n=0;cities[n].name;n++)
    continue;
  printf("%i largest cities in alphabetical order\n",n);
  if((list=calloc(n+1,sizeof(char*)))==NULL)
    {
    printf("can't allocate memory for list\n");
    exit(1);
    }
  for(n=0;cities[n].name;n++)
    list[n]=cities[n].name;
//qs0rt((void*)list,n,sizeof(char*),CompareStrings);
  qsort((void*)list,n,sizeof(char*),CompareStrings);
  for(n=0;list[n];n++)
    printf("%s\n",list[n]);
  free(list);
  }

int CompareByCountry(const void*a,const void*b)
  {
  int k;
  k=_stricmp(((CITY*)a)->country,((CITY*)b)->country);
  if(k!=0)
    return(k);
  return(_stricmp(((CITY*)a)->name,((CITY*)b)->name));
  }

void ByCountry()
  {
  int n;
  for(n=0;cities[n].name;n++)
    continue;
  printf("%i largest cities by country\n",n);
  qsort((void*)cities,n,sizeof(CITY),CompareByCountry);
  for(n=0;cities[n].name;n++)
    printf("%s, %s\n",cities[n].name,cities[n].country);
  }

int CompareByCountryAndPopulation(const void*a,const void*b)
  {
  int k;
  k=_stricmp(((CITY*)a)->country,((CITY*)b)->country);
  if(k!=0)
    return(k);
  k=(((CITY*)a)->population)-(((CITY*)b)->population);
  if(k!=0)
    return(k);
  return(_stricmp(((CITY*)a)->name,((CITY*)b)->name));
  }

void ByCountryAndPopulation()
  {
  int n;
  for(n=0;cities[n].name;n++)
    continue;
  printf("%i largest cities by country\n",n);
  qsort((void*)cities,n,sizeof(CITY),CompareByCountryAndPopulation);
  for(n=0;cities[n].name;n++)
    printf("%s, %s, %i\n",cities[n].name,cities[n].country,cities[n].population);
  }

typedef struct{double temperature,time;}DATA;

DATA data[]={
  { 6,44197.41667},
  { 7,44197.25   },
  { 9,44197.08333},
  {10,44198.25   },
  {10,44198.41667},
  {12,44198.08333},
  {17,44197.58333},
  {19,44197.91667},
  {23,44198.58333},
  {25,44199.25   },
  {26,44197.75   },
  {28,44199.08333},
  {29,44198.91667},
  {30,44198.75   }};

int CompareData(const void*v1,const void*v2)
  {
  if((*(DATA*)v1).time<(*(DATA*)v2).time)
    return(-1);
  if((*(DATA*)v1).time>(*(DATA*)v2).time)
    return(1);
  return(0);
  }

void SortTemperatures()
  {
  int n=sizeof(data)/sizeof(DATA);
  printf("%i temperatures by time\n",n);
  qsort((void*)data,n,sizeof(DATA),CompareData);
  for(n=0;n<sizeof(data)/sizeof(DATA);n++)
    printf("%11.5lf %3.0lf\n",data[n].time,data[n].temperature);
  }

typedef struct{double X,Y,Z;}NODE;

NODE nodes[]={
  { 0.0322,0.7164,0.0615},
  { 0.0345,0.7167,0.0615},
  { 0.0327,0.7251,0.0618},
  { 0.0364,0.7196,0.0615},
  { 0.03  ,0.7182,0.0615},
  { 0.0373,0.724 ,0.0615},
  { 0.0287,0.7218,0.0615},
  { 0.0369,0.7284,0.0615},
  {-0.0327,0.7251,0.0618},
  {-0.0287,0.7218,0.0615},
  {-0.0284,0.7262,0.0615},
  {-0.0293,0.7305,0.0615},
  {-0.03  ,0.7182,0.0615},
  {-0.0311,0.7335,0.0615}};

int CompareNodes(const void*v1,const void*v2)
  {
  if((*(NODE*)v1).X<(*(NODE*)v2).X)
    return(-1);
  if((*(NODE*)v1).X>(*(NODE*)v2).X)
    return(1);
  if((*(NODE*)v1).Y<(*(NODE*)v2).Y)
    return(-1);
  if((*(NODE*)v1).Y>(*(NODE*)v2).Y)
    return(1);
  if((*(NODE*)v1).Z<(*(NODE*)v2).Z)
    return(-1);
  if((*(NODE*)v1).Z>(*(NODE*)v2).Z)
    return(1);
  return(0);
  }

void SortNodes()
  {
  int n=sizeof(nodes)/sizeof(NODE);
  printf("%i nodes by XYZ\n",n);
  qsort((void*)nodes,n,sizeof(NODE),CompareNodes);
  for(n=0;n<sizeof(nodes)/sizeof(NODE);n++)
    printf("%7.4lf %6.4lf %6.4lf\n",nodes[n].X,nodes[n].Y,nodes[n].Z);
  }

int main(int argc,char**argv,char**envp)
  {
//JustNames();
//ByCountry();
//ByCountryAndPopulation();
//SortTemperatures();
  SortNodes();
  return(0);
  }
